/*
 * oss_api.c -- useful functions to record playback using oss driver
 *
 * author: hackbard@hackdaworld.dyndns.org
 *
 */

#include <stdio.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/soundcard.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>

#include "oss_api.h"

int get_dsp_cap(int fd,dsp_set *set,unsigned char print) {
	if(ioctl(fd,SNDCTL_DSP_GETCAPS,&(set->cap))==-1) {
		perror("SNDCTL_DSP_GETCAPS");
		return -1;
	}
	
	if(print) {
		printf("device capabilities:\n\n");
		printf("revision: %x\n",set->cap&DSP_CAP_REVISION);
		printf("full duplex:\t%c\n",set->cap&DSP_CAP_DUPLEX?'y':'n');
		printf("real time:\t%c\n",set->cap&DSP_CAP_REALTIME?'y':'n');
		printf("int. buf:\t%c\n",set->cap&DSP_CAP_BATCH?'y':'n');
		printf("coprociessor:\t%c\n",set->cap&DSP_CAP_COPROC?'y':'n');
		printf("trigger:\t%c\n",set->cap&DSP_CAP_TRIGGER?'y':'n');
		printf("support mmap:\t%c\n",set->cap&DSP_CAP_MMAP?'y':'n');
		printf("multiple open:\t%c\n",set->cap&DSP_CAP_MULTI?'y':'n');
		printf("chan binding:\t%c\n",set->cap&DSP_CAP_BIND?'y':'n');
	}
		
	return 1;
}

int set_dsp(int fd,dsp_set *set) {
	int tmp;

	tmp=set->format;
	if(ioctl(fd,SNDCTL_DSP_SETFMT,&(set->format))==-1) {
		perror("SNDCTL_DSP_SETFMT");
		return -1;
	}
	if(set->format!=tmp) {
		printf("format (%d) not supported!\n",tmp);
		return -1;
	}
	tmp=set->channel;
	if(ioctl(fd,SNDCTL_DSP_CHANNELS,&(set->channel))==-1) {
		perror("SNDCTL_DSP_CHANNELS");
		return -1;
	}
	if(set->channel!=tmp) {
		printf("%d-channel use not supported!\n",tmp);
		return -1;
	}
	tmp=set->freq;
	if(ioctl(fd,SNDCTL_DSP_SPEED,&(set->freq))==-1) {
		perror("SNDCTL_DSP_SPEED");
		return -1;
	}
	if(set->freq!=tmp) {
		printf("%d hz not suported!\n",tmp);
		return -1;
	}
	if(ioctl(fd,SNDCTL_DSP_GETBLKSIZE,&(set->bufsize))==-1) {
		perror("SOUNDCTL_DSP_GETBLKSIZE");
		return -1;
	}
	printf("\n\nSOUNDCTL_DSP_GETBLKSIZE = %d\n\n",set->bufsize);

	return 1;
}

int open_sound_dev(char *dev,int mode) {
	int fd;

	if((fd=open(dev,mode))==-1) {
		perror("open");
		return -1;
	}

	return fd;
}

int open_file(char *file,int mode) {
	int fd;
	
	if(mode) {
		if((fd=open(file,mode,S_IRWXU))==-1) {
			perror("open");
			return -1;
		}
	} else {
		if((fd=open(file,mode))==-1) {
			perror("open");
			return -1;
		}
	}

	return fd;
}

int close_it(int fd) {
	if(close(fd)==-1) {
		perror("close");
		return -1;
	}
	
	return 1;
}

